@@ -21,14 +21,12 @@
 #include <config.h>
 #endif
 
+#include <stdio.h>
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#ifdef HAVE_STDIO_H
-#include <stdio.h>
-#endif
-
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -41,9 +39,14 @@
 #include <fcntl.h>
 #endif
 
-
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+
 #include <glib.h>
 
 #if defined(GETTEXT_PACKAGE)
@@ -60,7 +63,10 @@ static gboolean version = FALSE;
 static gboolean list = FALSE;
 static gboolean verbose = FALSE;
 static gboolean create = FALSE;
-static gboolean remove = FALSE;
+static gboolean reset = FALSE;
+static gboolean recursive = FALSE;
+static gboolean force_array = FALSE;
+static gboolean monitor = FALSE;
 static gchar *channel_name = NULL;
 static gchar *property_name = NULL;
 static gchar **set_value = NULL;
@@ -68,12 +74,39 @@ static gchar **type = NULL;
 static gchar *import_file = NULL;
 static gchar *export_file = NULL;
 
+static void
+xfconf_query_monitor (XfconfChannel *channel, const gchar *changed_property, GValue *property_value)
+{
+    gchar *string;
+    
+    if (property_name && !g_str_has_prefix (changed_property, property_name))
+        return;
+
+    if (property_value && G_IS_VALUE (property_value))
+    {
+        if (verbose)
+        {
+           string = _xfconf_string_from_gvalue (property_value);
+           g_print (_("Property '%s' changed: %s\n"), changed_property, string);
+           g_free (string);
+       }
+       else
+       {
+           g_print (_("Property '%s' changed\n"), changed_property);
+       }
+    }
+    else
+    {
+        g_print (_("Property '%s' removed\n"), changed_property);
+    }
+}
+
 static gboolean
 xfconf_query_import_channel (XfconfChannel *channel, gint fd, GError **error)
 {
     if (error != NULL)
     {
-        *error = g_error_new (G_FILE_ERROR, 1, "method not implemented");
+        g_set_error (error, G_FILE_ERROR, 1, _("Export method not yet implemented"));
     }
     else
     {
@@ -87,7 +120,7 @@ xfconf_query_export_channel (XfconfChannel *channel, gint fd, GError **error)
 {
     if (error != NULL)
     {
-        *error = g_error_new (G_FILE_ERROR, 1, "method not implemented");
+        g_set_error (error, G_FILE_ERROR, 1, _("Export not yet implemented"));
     }
     else
     {
@@ -108,29 +141,37 @@ xfconf_query_get_propname_size (gpointer key, gpointer value, gpointer user_data
 }
 
 static void
-xfconf_query_list_contents (gpointer key, gpointer value, gpointer user_data)
+xfconf_query_list_sorted (gpointer key, gpointer value, gpointer user_data)
 {
-    gint i;
-    gint size = *(gint *)user_data;
-    gint property_name_size = 0, value_size = 0;
-    gchar *property_name = (gchar *)key;
-    GValue *property_value = (GValue *)value;
+  GSList **list = user_data;
+  
+  *list = g_slist_insert_sorted (*list, key, (GCompareFunc) g_utf8_collate);
+}
 
-    if (verbose)
+static void
+xfconf_query_list_contents (GSList *sorted_contents, GHashTable *channel_contents, gint size)
+{
+    GSList *li;
+    gchar *format = verbose ? g_strdup_printf ("%%-%ds%%s\n", size + 2) : NULL;
+    GValue *property_value;
+    gchar *string;
+    
+    for (li = sorted_contents; li != NULL; li = li->next)
     {
-        g_print("%s%n", property_name, &property_name_size);
-        for(i = property_name_size; i < (size+2); ++i)
+        if (verbose)
         {
-            g_print(" ");
+            property_value = g_hash_table_lookup (channel_contents, li->data);
+            string = _xfconf_string_from_gvalue (property_value);
+            g_print (format, (gchar *) li->data, string);
+            g_free (string);
+        }
+        else
+        {
+            g_print ("%s\n", (gchar *) li->data);
         }
-        const gchar *str_val = _xfconf_string_from_gvalue(property_value);
-        g_print("%s%n", str_val, &value_size);
-        g_print("\n");
-    }
-    else
-    {
-        g_print ("%s\n", property_name);
     }
+    
+    g_free (format);
 }
 
 static GOptionEntry entries[] =
@@ -140,19 +181,19 @@ static GOptionEntry entries[] =
         NULL
     },
     {    "channel", 'c', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &channel_name,
-        N_("pick the channel"),
+        N_("Pick the channel"),
         NULL
     },
     {    "property", 'p', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &property_name,
-        N_("pick the property"),
+        N_("Pick the property"),
         NULL
     },
     {    "set", 's', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING_ARRAY, &set_value,
-        N_("set (change the value)"),
+        N_("Change or set the value of the property"),
         NULL
     },
     {    "list", 'l', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &list,
-        N_("List properties"),
+        N_("List properties (or channels if -c isn't specified)"),
         NULL
     },
     {    "verbose", 'v', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &verbose,
@@ -167,16 +208,28 @@ static GOptionEntry entries[] =
        N_("Specify the property value type"),
        NULL
     },
-    {    "remove", 'r', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &remove,
-        N_("Remove property"),
+    {    "reset", 'r', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &reset,
+        N_("Reset property"),
+        NULL
+    },
+    {    "recursive", 'R', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &recursive,
+        N_("Recursive (use with -r)"),
+        NULL
+    },
+    {   "force-array", 'a', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &force_array,
+        N_("Force array even if only one element"),
         NULL
     },
     {   "export", 'x', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &export_file,
-        N_("Export channel"),
+        N_("Export channel to file"),
         NULL,
     },
     {   "import", 'i', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &import_file,
-        N_("Import channel"),
+        N_("Import channel from file"),
+        NULL,
+    },
+    {   "monitor", 'm', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &monitor,
+        N_("Monitor a channel for property changes"),
         NULL,
     },
     { NULL }
@@ -192,7 +245,12 @@ main(int argc, char **argv)
     gint fd = -1;
 
     g_type_init();
-    xfconf_init(NULL);
+    if(!xfconf_init(&error))
+    {
+        g_critical("Failed to init libxfconf: %s\n", error->message);
+        g_error_free(error);
+        return 1;
+    }
     
     GOptionContext *context = g_option_context_new("- xfconf commandline utility");
 
@@ -219,29 +277,32 @@ main(int argc, char **argv)
         g_print("Channels:\n");
 
         channels = xfconf_list_channels();
-        for(i = 0; channels[i]; ++i)
-            g_print("  %s\n", channels[i]);
-        g_strfreev(channels);
+        if(G_LIKELY(channels)) {
+            for(i = 0; channels[i]; ++i)
+                g_print("  %s\n", channels[i]);
+            g_strfreev(channels);
+        } else
+            return 1;
 
         return 0;
     }
 
     /** Check if the property is specified */
-    if(!property_name && !list && !export_file && !import_file)
+    if(!property_name && !list && !export_file && !import_file && !monitor)
     {
         g_print("No property specified, aborting...\n");
         return 1;
     }
 
-    if (create && remove)
+    if (create && reset)
     {
-        g_print("--create and --remove options can not be used together,\naborting...\n");
+        g_print("--create and --reset options can not be used together,\naborting...\n");
         return 1;
     }
 
-    if ((create || remove) && (list))
+    if ((create || reset) && (list))
     {
-        g_print("--create and --remove options can not be used together with\n --list\naborting...\n");
+        g_print("--create and --reset options can not be used together with\n --list\naborting...\n");
         return 1;
     }
 
@@ -251,20 +312,33 @@ main(int argc, char **argv)
         return 1;
     }
 
-    if ((import_file || export_file) && (list || property_name || create || remove))
+    if ((import_file || export_file) && (list || property_name || create || reset))
     {
-        g_print("--import and --export options can not be used together with\n --create, --remove, --property and --list,\naborting...\n");
+        g_print("--import and --export options can not be used together with\n --create, --reset, --property and --list,\naborting...\n");
         return 1;
     }
 
     channel = xfconf_channel_new(channel_name);
-
-    if (property_name)
+    
+    if (monitor)
     {
-        /** Remove property */
-        if (remove)
+        g_signal_connect (G_OBJECT (channel), "property-changed", G_CALLBACK (xfconf_query_monitor), NULL);
+        
+        g_print ("\n");
+        g_print (_("Start monitoring channel '%s':"), channel_name);
+        g_print ("\n------------------------------------------------\n\n");
+        
+        GMainLoop *loop = g_main_loop_new (NULL, TRUE);
+        g_main_loop_run (loop);
+        g_main_loop_unref (loop);        
+    }
+
+    if (property_name && !list)
+    {
+        /** Reset property */
+        if (reset)
         {
-            xfconf_channel_remove_property(channel, property_name);
+            xfconf_channel_reset_property(channel, property_name, recursive);
         }
         /** Read value */
         else if(set_value == NULL || set_value[0] == NULL)
@@ -336,7 +410,7 @@ main(int argc, char **argv)
                 }
             }
 
-            if(!set_value[1])
+            if(!set_value[1] && !force_array)
             {
                 /* not an array */
                 GType gtype = G_TYPE_INVALID;
@@ -451,12 +525,20 @@ main(int argc, char **argv)
 
     if (list)
     {
-        GHashTable *channel_contents = xfconf_channel_get_properties(channel, NULL);
+        GHashTable *channel_contents = xfconf_channel_get_properties(channel, property_name);
         if (channel_contents)
         {
             gint size = 0;
-            g_hash_table_foreach (channel_contents, (GHFunc)xfconf_query_get_propname_size, &size);
-            g_hash_table_foreach (channel_contents, (GHFunc)xfconf_query_list_contents, &size);
+            if (verbose)
+                g_hash_table_foreach (channel_contents, (GHFunc)xfconf_query_get_propname_size, &size);
+            
+            GSList *sorted_contents = NULL;
+            g_hash_table_foreach (channel_contents, (GHFunc)xfconf_query_list_sorted, &sorted_contents);
+            
+            xfconf_query_list_contents (sorted_contents, channel_contents, size);
+            
+            g_slist_free (sorted_contents);
+            g_hash_table_destroy (channel_contents);
         }
         else
         {
@@ -468,50 +550,58 @@ main(int argc, char **argv)
     {
         if (!strcmp(export_file, "-"))
         {
-            /* Open stdout */
-            xfconf_query_export_channel (channel, 1, NULL);
+            /* Use stdout */
+            fd = fileno (stdout);
         }
         else
         {
-            fd = open (export_file, O_CREAT | O_EXCL, 0);
+            fd = open (export_file, O_CREAT | O_EXCL | O_WRONLY, 0);
             if (fd < 0)
             {
-                g_print ("Could not create export-file '%s',\n aborting...\n", export_file);
+                g_printerr (_("Could not create export file \"%s\": %s\n"), export_file, strerror (errno));
                 return 1;
             }
-            if (!xfconf_query_export_channel (channel, fd, &error))
-            {
-                close (fd);
-                g_print ("Could not create export-file '%s',\n Cause: '%s'\n", export_file, error->message);
-                return 1;
-            }
-            close (fd);
         }
+
+        if (!xfconf_query_export_channel (channel, fd, &error))
+        {
+            if (fd != fileno (stdout))
+                close (fd);
+            g_printerr (_("Could not create export file \"%s\": %s\n"), export_file, error->message);
+            g_error_free (error);
+            return 1;
+        }
+        if (fd != fileno (stdout))
+            close (fd);
     }
 
     if (import_file)
     {
         if (!strcmp(import_file, "-"))
         {
-            /* Open stdin */
-            xfconf_query_import_channel (channel, 0, NULL);
+            /* Use stdin */
+            fd = fileno (stdin);
         }
         else
         {
             fd = open (import_file, O_RDONLY, 0);
             if (fd < 0)
             {
-                g_print ("Could not open import-file '%s',\n aborting...\n", import_file);
+                g_printerr (_("Could not open import file \"%s\": %s\n"), import_file, strerror (errno));
                 return 1;
             }
-            if (!xfconf_query_import_channel (channel, fd, &error))
-            {
-                close (fd);
-                g_print ("Could not parse import-file '%s',\n Cause: '%s'\n", import_file, error->message);
-                return 1;
-            }
-            close (fd);
         }
+
+        if (!xfconf_query_import_channel (channel, fd, &error))
+        {
+            if (fd != fileno (stdin))
+                close (fd);
+            g_printerr (_("Could not parse import file \"%s\": %s\n"), import_file, error->message);
+            g_error_free (error);
+            return 1;
+        }
+        if (fd != fileno (stdin))
+            close (fd);
     }
 
     return 0;
