@@ -28,10 +28,12 @@ typedef struct
 
 DBusGConnection *_xfconf_get_dbus_g_connection();
 DBusGProxy *_xfconf_get_dbus_g_proxy();
-DBusGProxy *_xfconf_get_gui_dbus_g_proxy();
 
 XfconfNamedStruct *_xfconf_named_struct_lookup(const gchar *struct_name);
 
 void _xfconf_channel_shutdown();
 
+void _xfconf_g_bindings_init();
+void _xfconf_g_bindings_shutdown();
+
 #endif  /* __XFCONF_PRIVATE_H__ */
