@@ -37,7 +37,6 @@
 static guint xfconf_refcnt = 0;
 static DBusGConnection *dbus_conn = NULL;
 static DBusGProxy *dbus_proxy = NULL;
-static DBusGProxy *gui_dbus_proxy = NULL;
 static GHashTable *named_structs = NULL;
 
 
@@ -65,17 +64,6 @@ _xfconf_get_dbus_g_proxy()
     return dbus_proxy;
 }
 
-DBusGProxy *
-_xfconf_get_gui_dbus_g_proxy()
-{
-    if(!xfconf_refcnt) {
-        g_critical("xfconf_init() must be called before attempting to use libxfconf!");
-        return NULL;
-    }
-
-    return gui_dbus_proxy;
-}
-
 XfconfNamedStruct *
 _xfconf_named_struct_lookup(const gchar *struct_name)
 {
@@ -91,6 +79,33 @@ _xfconf_named_struct_free(XfconfNamedStruct *ns)
 
 
 
+static void
+xfconf_static_dbus_init()
+{
+    static gboolean static_dbus_inited = FALSE;
+
+    if(!static_dbus_inited) {
+        dbus_g_error_domain_register(XFCONF_ERROR, "org.xfce.Xfconf.Error",
+                                     XFCONF_TYPE_ERROR);
+
+        dbus_g_object_register_marshaller(_xfconf_marshal_VOID__STRING_STRING_BOXED,
+                                          G_TYPE_NONE,
+                                          G_TYPE_STRING,
+                                          G_TYPE_STRING,
+                                          G_TYPE_VALUE,
+                                          G_TYPE_INVALID);
+        dbus_g_object_register_marshaller(_xfconf_marshal_VOID__STRING_STRING,
+                                          G_TYPE_NONE,
+                                          G_TYPE_STRING,
+                                          G_TYPE_STRING,
+                                          G_TYPE_INVALID);
+
+        static_dbus_inited = TRUE;
+    }
+}
+
+
+
 /* public api */
 
 /**
@@ -113,8 +128,7 @@ xfconf_init(GError **error)
 
     g_type_init();
 
-    dbus_g_error_domain_register(XFCONF_ERROR, "org.xfce.Xfconf.Error",
-                                 XFCONF_TYPE_ERROR);
+    xfconf_static_dbus_init();
 
     dbus_conn = dbus_g_bus_get(DBUS_BUS_SESSION, error);
     if(!dbus_conn)
@@ -125,28 +139,14 @@ xfconf_init(GError **error)
                                            "/org/xfce/Xfconf",
                                            "org.xfce.Xfconf");
 
-    dbus_g_object_register_marshaller((GClosureMarshal)xfconf_marshal_VOID__STRING_STRING_BOXED,
-                                      G_TYPE_NONE,
-                                      G_TYPE_STRING,
-                                      G_TYPE_STRING,
-                                      G_TYPE_VALUE,
-                                      G_TYPE_INVALID);
     dbus_g_proxy_add_signal(dbus_proxy, "PropertyChanged",
                             G_TYPE_STRING, G_TYPE_STRING, G_TYPE_VALUE,
                             G_TYPE_INVALID);
-    dbus_g_object_register_marshaller((GClosureMarshal)xfconf_marshal_VOID__STRING_STRING,
-                                      G_TYPE_NONE,
-                                      G_TYPE_STRING,
-                                      G_TYPE_STRING,
-                                      G_TYPE_INVALID);
     dbus_g_proxy_add_signal(dbus_proxy, "PropertyRemoved",
                             G_TYPE_STRING, G_TYPE_STRING,
                             G_TYPE_INVALID);
 
-    gui_dbus_proxy = dbus_g_proxy_new_for_name(dbus_conn,
-                                               "org.xfce.Xfconf",
-                                               "/org/xfce/Xfconf",
-                                               "org.xfce.Xfconf.GUI");
+    _xfconf_g_bindings_init();
 
     ++xfconf_refcnt;
     return TRUE;
@@ -171,6 +171,7 @@ xfconf_shutdown()
         return;
     }
 
+    _xfconf_g_bindings_shutdown();
     _xfconf_channel_shutdown();
 
     if(named_structs) {
@@ -181,9 +182,6 @@ xfconf_shutdown()
     g_object_unref(G_OBJECT(dbus_proxy));
     dbus_proxy = NULL;
 
-    g_object_unref(G_OBJECT(gui_dbus_proxy));
-    gui_dbus_proxy = NULL;
-
     dbus_g_connection_unref(dbus_conn);
     dbus_conn = NULL;
 
