@@ -27,6 +27,10 @@
 #include <signal.h>
 #endif
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -85,8 +89,9 @@ signal_pipe_io(GIOChannel *source,
     guint32 sigstate = 0;
     gsize bread = 0;
     
-    if(G_IO_ERROR_NONE == g_io_channel_read(source, (gchar *)&sigstate,
-                                            sizeof(sigstate), &bread)
+    if(G_IO_STATUS_NORMAL == g_io_channel_read_chars(source, (gchar *)&sigstate,
+                                                     sizeof(sigstate), &bread,
+                                                     NULL)
        && sizeof(sigstate) == bread)
     {
         switch(sigstate)
@@ -123,9 +128,9 @@ main(int argc,
     gchar **backends = NULL;
     gboolean print_version = FALSE;
     GOptionEntry options[] = {
-        { "version", 'V', 0, G_OPTION_ARG_NONE, NULL,
+        { "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
             N_("Prints the xfconfd version"), NULL },
-        { "backends", 'b', 0, G_OPTION_ARG_STRING_ARRAY, NULL,
+        { "backends", 'b', 0, G_OPTION_ARG_STRING_ARRAY, &backends,
             N_("Configuration backends to use.  The first backend specified " \
                "is opened read/write; the others, read-only."), NULL },
         { NULL, 0, 0, 0, 0, NULL, NULL },
@@ -146,30 +151,27 @@ main(int argc,
     
     xfce_textdomain(PACKAGE, LOCALEDIR, "UTF-8");
     
-    g_set_application_name("Xfce Configuration Daemon");
-    g_set_prgname("xfconfd");
+    g_set_application_name(_("Xfce Configuration Daemon"));
+    g_set_prgname(G_LOG_DOMAIN);
     g_type_init();
     
-    options[0].arg_data = &print_version;
-    options[1].arg_data = &backends;
-    
     opt_ctx = g_option_context_new(NULL);
     g_option_context_set_translation_domain(opt_ctx, PACKAGE);
-    g_option_context_set_summary(opt_ctx, N_("Xfce configuration daemon"));
+    g_option_context_set_summary(opt_ctx, _("Xfce configuration daemon"));
     g_option_context_set_description(opt_ctx,
-                                     N_("Report bugs to http://bugs.xfce.org/\n"));
+                                     _("Report bugs to http://bugs.xfce.org/\n"));
     g_option_context_add_main_entries(opt_ctx, options, PACKAGE);
     if(!g_option_context_parse(opt_ctx, &argc, &argv, &error)) {
-        g_printerr("Error parsing options: %s\n", error->message);
+        g_printerr(_("Error parsing options: %s\n"), error->message);
         g_error_free(error);
         g_option_context_free(opt_ctx);
-        return 1;
+        return EXIT_FAILURE;
     }
     g_option_context_free(opt_ctx);
     
     if(print_version) {
-        g_print("Xfconfd version " VERSION "\n");
-        return 0;
+        g_print("Xfconfd " VERSION "\n");
+        return EXIT_SUCCESS;
     }
     
     mloop = g_main_loop_new(NULL, FALSE);
@@ -201,9 +203,9 @@ main(int argc,
     
     xfconfd = xfconf_daemon_new_unique(backends, &error);
     if(!xfconfd) {
-        g_printerr("Xfconfd failed to start: %s\n", error->message);
+        g_critical("Xfconfd failed to start: %s\n", error->message);
         g_error_free(error);
-        return 1;
+        return EXIT_FAILURE;
     }
     g_strfreev(backends);
     
@@ -219,5 +221,5 @@ main(int argc,
     
     g_main_loop_unref(mloop);
     
-    return 0;
+    return EXIT_SUCCESS;
 }
