@@ -357,32 +357,36 @@ xfconf_backend_exists(XfconfBackend *backend,
 }
 
 /**
- * xfconf_backend_remove:
+ * xfconf_backend_reset:
  * @backend: The #XfconfBackend.
  * @channel: A channel name.
  * @property: A property name.
- * @recursive: Whether or not the remove is recursive.
+ * @recursive: Whether or not the reset is recursive.
  * @error: An error return.
  *
- * Removes the property identified by @property from @channel.
+ * Resets the property identified by @property from @channel.
  * If @recursive is %TRUE, all sub-properties of @property will be
- * removed as well.  If the empty string ("") or a forward slash ("/")
- * is specified for @property, the entire channel will be removed.
+ * reset as well.  If the empty string ("") or a forward slash ("/")
+ * is specified for @property, the entire channel will be reset.
+ *
+ * If none of the properties specified are locked or have root-owned
+ * system-wide defaults set, this effectively removes the properties
+ * from the configuration store entirely.
  *
  * Return value: The backend should return %TRUE if the operation
  *               was successful, or %FALSE otherwise.  On %FALSE,
  *               @error should be set to a description of the failure.
  **/
 gboolean
-xfconf_backend_remove(XfconfBackend *backend,
-                      const gchar *channel,
-                      const gchar *property,
-                      gboolean recursive,
-                      GError **error)
+xfconf_backend_reset(XfconfBackend *backend,
+                     const gchar *channel,
+                     const gchar *property,
+                     gboolean recursive,
+                     GError **error)
 {
     XfconfBackendInterface *iface = XFCONF_BACKEND_GET_INTERFACE(backend);
     
-    xfconf_backend_return_val_if_fail(iface && iface->remove && channel
+    xfconf_backend_return_val_if_fail(iface && iface->reset && channel
                                       && *channel && property && *property
                                       && (!error || !*error), FALSE);
     if(!xfconf_channel_is_valid(channel, error))
@@ -391,7 +395,7 @@ xfconf_backend_remove(XfconfBackend *backend,
     if(!recursive && (!*property || (property[0] == '/' && !property[1]))) {
         if(error) {
             g_set_error(error, XFCONF_ERROR, XFCONF_ERROR_INVALID_PROPERTY,
-                        _("The property name can only be empty or \"/\" if a recursive remove was specified."));
+                        _("The property name can only be empty or \"/\" if a recursive reset was specified."));
         }
         return FALSE;
     }
@@ -401,13 +405,14 @@ xfconf_backend_remove(XfconfBackend *backend,
         return FALSE;
     }
 
-    return iface->remove(backend, channel, property, recursive, error);
+    return iface->reset(backend, channel, property, recursive, error);
 }
 
 /**
  * xfconf_backend_list_channels:
  * @backend: The #XfconfBackend.
  * @channels: A pointer to a #GSList head.
+ * @error: An error return.
  *
  * Instructs the backend to return a list of channels with
  * configuration data stored in the configuration store.
@@ -429,6 +434,40 @@ xfconf_backend_list_channels(XfconfBackend *backend,
     return iface->list_channels(backend, channels, error);
 }
 
+/**
+ * xfconf_backend_is_property_locked:
+ * @backend: The #XfconfBackend.
+ * @channel: A channel name.
+ * @property: A property name.
+ * @locked: A boolean return.
+ * @error: An error return.
+ *
+ * Queries whether or not the property on @channel is locked by
+ * system policy.
+ *
+ * Return value: The backend should return %TRUE if the operation
+ *               was successful, or %FALSE otherwise.  On %FALSE,
+ *               @error should be set to a description of the failure.
+ **/
+gboolean
+xfconf_backend_is_property_locked(XfconfBackend *backend,
+                                  const gchar *channel,
+                                  const gchar *property,
+                                  gboolean *locked,
+                                  GError **error)
+{
+    XfconfBackendInterface *iface = XFCONF_BACKEND_GET_INTERFACE(backend);
+
+    xfconf_backend_return_val_if_fail(iface && iface->is_property_locked
+                                      && (!error || !*error), FALSE);
+    if(!xfconf_channel_is_valid(channel, error))
+        return FALSE;
+    if(!xfconf_property_is_valid(property, error))
+        return FALSE;
+
+    return iface->is_property_locked(backend, channel, property, locked, error);
+}
+
 /**
  * xfconf_backend_flush
  * @backend: The #XfconfBackend.
